@@ -8,6 +8,7 @@ from django.db.models.signals import pre_save, post_save
 from djangoldp.models import Model
 from .permissions import CustomerPermissions, ProjectPermissions, ProjectMemberPermissions
 
+from .views import ProjectMembersViewset
 
 class Customer(Model):
     name = models.CharField(max_length=255)
@@ -49,6 +50,7 @@ def auto_increment_project_number():
   return last_inc.number + 1
 
 
+MODEL_MODIFICATION_USER_FIELD = 'modification_user'
 STATUS_CHOICES = [
     ('Public', 'Public'),
     ('Private', 'Private'),
@@ -89,31 +91,46 @@ class Project(Model):
 class Member(Model):
     project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='members')
     user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='projects')
-    name = models.CharField(max_length=255, null=True, blank=True)
     is_admin = models.BooleanField(default=False)
 
     class Meta(Model.Meta):
         container_path = "project-members/"
         permission_classes = [ProjectMemberPermissions]
         anonymous_perms = []
-        authenticated_perms = []
-        owner_perms = []
+        authenticated_perms = ['view', 'add']
+        owner_perms = ['inherit']
         unique_together = ['user', 'project']
+        rdf_type = 'hd:projectmember'
+        view_set = ProjectMembersViewset
 
     def __str__(self):
-        if self.name is None:
-            return ""
-        return self.name
+        return str(self.pk)
 
     def save(self, *args, **kwargs):
+        if self.user:
+            if self.user.username == "hubl-workaround-493":
+                self.user = None
+
         # cannot be duplicated Members
         if not self.pk and Member.objects.filter(project=self.project, user=self.user).exists():
-            # override existing member
-            existing = Member.objects.get(project=self.project, user=self.user)
-            existing.delete()
+            return
 
         super(Member, self).save(*args, **kwargs)
 
+@receiver(post_save, sender=Member)
+def fix_user_hubl_workaround_493(sender, instance, **kwargs):
+    if not instance.user:
+        try:
+            request_user = getattr(instance, MODEL_MODIFICATION_USER_FIELD, None)
+            if request_user is None:
+                raise Exception()
+            user = get_user_model().objects.get(pk=request_user.pk)
+            instance.user = user
+            instance.save()
+        except:
+            if instance.pk is not None:
+                instance.delete()
+
 
 @receiver(pre_save, sender=Project)
 def set_jabberid(sender, instance, **kwargs):
@@ -132,8 +149,11 @@ def set_jabberid(sender, instance, **kwargs):
 
 @receiver(post_save, sender=Project)
 def set_captain_as_member(instance, created, **kwargs):
-    # add captain as an admin member, if they've not already been added
-    if created and instance.captain is not None and\
-            not instance.members.filter(user=instance.captain).exists():
-        captain = Member(user=instance.captain, project=instance, is_admin=True)
-        captain.save()
+    if instance.captain is not None:
+        try:
+            captain_member = instance.members.get(user=instance.captain)
+            if not captain_member.is_admin:
+                captain_member.is_admin = True
+                captain_member.save()
+        except Member.DoesNotExist:
+            Member.objects.create(user=instance.captain, project=instance, is_admin=True)
